/*
 * cau.c
 * concat and uudecode
 * Version 2.2
 *
 * Usage: cau file1 file2 file3 file4 file5...
 *
 *    takes uuencoded files form the command line, throws out all lines
 * that are not uuencode format, and then passes each line to it's own
 * uudecode routine.
 *  files can be spread out, but MUST be specified in order on the command
 * line:
 *  EX: output from grep Subject: 23??
 *
 *           2345: Subject: fish1.gif [1/4]
 *           2346: Subject: fish1.gif [2/4]
 *           2347: Subject: fish1.gif [4/4]
 *           2348: Subject: fish1.gif [3/4]
 *           2349: Subject: cat.gif [2/3]
 *           2350: Subject: cat.gif [1/3]
 *           2351: Subject: cat.gif [3/3]
 *           2352: Subject: cat1.gif [1/1]
 *
 *    cau 2345 2346 2348 2347 2350 2349 2351 2352
 *      
 *   will create fish1.gif (actually was in the first 'begin' line)
 *  cat.gif and cat1.gif
 *
 *   Notice the order of the files on the command line.. in order of the
 *  parts.
 *  This will not create truncated decoded files as many of the other
 *  'split file decoders' will.
 *
 *  BUGS: - Some people seem to use a strange uuencode/spliter that puts
 *          'table' entries in the files.. Sometimes cau will fail on
 *          these files.
 *        - some people also use encoders that put a 'crc' on the end of
 *          the line. If it is not the 'r.e.m. encoder. then cau WILL fail
 *          on these files as this is NOT a standard uuencode format.
 *
 *  outdec function by someone named ajr (?)
 *
 * Have Fun.
 * Trevor Paquette       | Landmark/ITA
 * tpaquette@ita.lgc.com | Calgary, Alberta, Canada
 */

#include <stdio.h>

#define DEC(c)	(((c) - ' ') & 077)
#define SUMSIZE	64

FILE *out;
char **av, ac;
int havebegin, debug, isrem;
char caustrs[512][256];
int numcaustrs;

main(argc, argv)
int argc;
char **argv;
{
 FILE *in;
 int i,ind;
 
 ac = argc;
 av = argv;
 havebegin = 0;
 debug = 0;
 ind = 1;

 readcaurc();

 fprintf(stdout, "Decoding ");

 if(ac == 1) { /* Use stdin as input */
   in = stdin;
   caufile(in);
 }
 else if((ac == 2) && (!strcmp(av[1],"-d"))) {
   debug = 1;
   in = stdin;
   caufile(in);
 }
 else {
   for(i = ind; i < ac;i++ ) {
     if(!strcmp(av[i],"-d")) { /* debug */
       debug = 1;
     }
     else {
       if(!(in = fopen(av[i],"r"))) {
	 fprintf(stderr, "%s: %s: ", av[0], av[i]);
	 perror(in);
	 continue;
       } else {
	 caufile(in);
	 fclose(in);
       }
     }
   }
 }
 if(havebegin) {
   fclose(out);
   fprintf(stdout, "(Short file)"); fflush(stdout);
 }
 fprintf(stdout,"\n"); fflush(stdout);
 exit(0);
}

int caufile(in)
FILE *in;
{
 int mode, dec, slen, rest, flen, checksum;
 int remnum, i, doit, lenbuf;
 float remver;
 char remfile[256];
 char buf[256], dest[256], *bp;
 char t[256];

 while (fgets(buf, sizeof buf, in))  {

   lenbuf = strlen(buf);

   /* strip of trailing \n */
   if(buf[lenbuf-1] == '\n') {
     buf[lenbuf-1] = 0;
     lenbuf--;
   }

   /* strip of trailing ^M */
   if(buf[lenbuf-1] == 0x0d) {
     buf[lenbuf-1] = 0;
     lenbuf--;
   }

   /* only check line if isrem is not already set */
   if( (!isrem) && 
       (sscanf(buf,"section %d of uuencode %f of file %s by R.E.M.",
	       &remnum,&remver,remfile) == 3))
     isrem = 1;

   if(!strncmp(buf,"begin ",6) && isdigit(buf[6])) {

     if(havebegin) { /* file already opened, close file */
       fclose(out);
       fprintf(stdout, "(Short file)"); fflush(stdout);
       havebegin = 0;
     }

     havebegin = 1;			/* yes, we found valid begin line */
     if(sscanf(buf,"begin %o %s", &mode, dest) != 2) {
       fprintf(stderr, "%s: %s: Missing filename or mode\n", av[0], buf);
       continue;		/* try again when error occurs		*/
     }
     if((out = fopen(dest, "w")) == NULL) {	/* prepare outut file	*/
       fprintf(stderr, "%s: %s: ", av[0], dest);
       perror(in);
       continue;
     }
     chmod(dest, mode); /* set file-mode */
     fprintf(stdout, " %s", dest); fflush(stdout);
   }
   else if(!strncmp(buf, "end", 3)) {
     havebegin = 0;
     isrem = 0;
     fclose(out);
   }
   else if (havebegin) {
     /* assume line in valid format */
     doit = 1;
     /* some safe guards againest possible valid strings */
     for(i = 0 ; (doit) && (i < numcaustrs); i++) {
       if(!strncmp(buf,caustrs[i],strlen(caustrs[i]))) {
         doit = 0;
       }
     }
     if(doit) {
       dec = DEC(buf[0]);
       flen = dec / 3;
       rest = dec % 3;
       slen = flen * 4;
       if(rest) slen += 4;
       if(dec > 0) {
	 if(((lenbuf-1) == slen) || /* check for valid format */
	    ((lenbuf-2) == slen)) { /* check for unknown CRC check format  */
	   if(debug) {
	     fprintf(stderr,"\n%s, %d, %d",buf, slen, lenbuf-1);
	   }
	   bp = &buf[1];
	   while(dec > 0) {
	     outdec(bp, out, dec);
	     bp += 4;
	     dec -= 3;
	   }
	 }
       }
     }
   }
 }
}

/*
 * output a group of 3 bytes (4 input characters).
 * the input chars are pointed to by p, they are to
 * be output to file f.  n is used to tell us not to
 * output all of them at the end of the file.
 */
outdec(p, f, n)
char *p;
FILE *f;
int n;
{
  int c1, c2, c3;
  
  c1 = DEC(*p) << 2 | DEC(p[1]) >> 4;
  c2 = DEC(p[1]) << 4 | DEC(p[2]) >> 2;
  c3 = DEC(p[2]) << 6 | DEC(p[3]);

  if((n >= 1) && f) putc(c1, f);
  if((n >= 2) && f) putc(c2, f);
  if((n >= 3) && f) putc(c3, f);
  return((c1+c2+c3) % SUMSIZE);
}

/* check checksum on the line */
issummed(buf)
char *buf;
{
 int n,checksum;
 char *bp;

 /* I cannot remember who wrote or where I got this from.. */
 checksum = 0;
 n = DEC(buf[0]);
 if(n <= 0) return(0);

 bp = &buf[1];
 while (n > 0) {
   checksum = (checksum+outdec(bp, 0, n)) % SUMSIZE;
   bp += 4;
   n -= 3;
 }
 
 /* if checksum matches.. return */
 if(checksum == DEC(*bp))
   return(checksum);

 return(0);
}

readcaurc()
{
 FILE *fp;
 char caurcfile[512], str[256];
 char *home, *getenv();
 
 fp = 0;
 home = getenv("HOME");
 if(home) {
   sprintf(caurcfile,"%s/.caurc",home);
   fp = fopen(caurcfile,"r");
 }
 if(!fp) {
   sprintf(caurcfile,".caurc");
   fp = fopen(caurcfile,"r");
   if(!fp) {
     numcaustrs = 0;
     /* use internal defaults */
     strcpy(caustrs[0], "END-");
     strcpy(caustrs[1], "END ");
     strcpy(caustrs[2], "From:");
     strcpy(caustrs[3], "Date:");
     strcpy(caustrs[4], "BEGIN-");
     strcpy(caustrs[5], "BEGIN ");
     strcpy(caustrs[6], "To find");
     strcpy(caustrs[7], "Reply-To");
     strcpy(caustrs[8], "Sam Sheddan");
     strcpy(caustrs[9], "your system");
     strcpy(caustrs[10], "X-Received:");
     strcpy(caustrs[11], "Newsgroups:");
     strcpy(caustrs[12], "Originator:");
     strcpy(caustrs[13], "Organization:");
     strcpy(caustrs[14], "	>>> Processed using");
     strcpy(caustrs[15], "Subject:");
     strcpy(caustrs[16], "part ");
     strcpy(caustrs[17], "CUT HERE");
     strcpy(caustrs[18], "echo ");
     strcpy(caustrs[19], "This is");
     strcpy(caustrs[20], "when it");
     strcpy(caustrs[21], "Once again");
     strcpy(caustrs[22], "derriere.");
     numcaustrs = 23;
     return(1);
   }
 }

 fgets(str,256,fp);
 while(str[0] != 0) {
   str[strlen(str)-1] = 0;
   strcpy(caustrs[numcaustrs], str);
   numcaustrs++;
   str[0] = 0;
   fgets(str,256,fp);
 }
 return(1);
}
