#include <stdio.h>
#include <sys/types.h>
#include <sys/time.h>

enum {BOUNCE, ROLL_LEFT, ROLL_RIGHT, RANDOM, N_MODES};

char * led_progname;

void led_error (s1, s2)
char * s1, s2;
{
   fprintf (stderr, "%s: %s%s\n", led_progname, s1, s2);
   exit (-1);
}

void light (i)
int i;
{
   printf ("\033[%iq", i);
}

void flash (i)
int i;
{
   printf ("\033[?%it", i);
}

void clear ()
{
   printf ("\033[0q\033[?0t");
}

void load (v)
int v;
{
   int i, y;
   y=v;
   clear();
   for (i=4; i>0; i--)
   {
      switch (y % 10)
      {
      case 0: /* dark */
	 /* do nothing */
	 break;
      case 1: /* lit */
	 light (i);
	 break;
      case 2: /* flashing */
	 flash (i);
	 break;
      }
      y /= 10;
   }
}

void bounce (t)
unsigned int t;
{
   int i;
   unsigned int s;
   s = t / 6;
   for (i=1; i<4; i++)
   {
      clear();
      light (i);
      fflush (stdout);
      usleep (s);
   }
   for (i=4; i>1; i--)
   {
      clear();
      light (i);
      fflush (stdout);
      usleep (s);
   }
}

void roll (t, min, max, step)
unsigned int t;
int min, max, step;
{
   int i;
   unsigned int s;
   s = t / ((abs (max - min)) / abs (step));
   for (i = min; i != max; i += step)
   {
      clear();
      light (i);
      fflush (stdout);
      usleep (s);
   }
}

void random (t)
unsigned int t;
{
   int i, ran, mran;
   clear();
   ran = rand();
   for (i=1; i<5; i++)
   {
      mran = ran % 3;
      switch (mran)
      {
      case 0: /* dark */
	 /* do nothing -- cleared */
	 break;
      case 1: /* lit */
	 light (i);
	 break;
      case 2: /* flashing */
	 flash (i);
	 break;
      }
      ran /= 3;
   }
   fflush (stdout);
   usleep (t);
}

main(argc, argv)
int argc;
char ** argv;
{
   unsigned int time_between=1000000, argn, mode=0, random_mode = 0;
   led_progname = argv[0];
   srand (time(NULL));
   for (argn = 1; argn < argc && argv[argn][0] == '-'; argn++)
   {
      switch (argv[argn][1])
      {
      case 't':
	 time_between = 1000 * atoi(argv[++argn]);
	 break;
      case 'm':
	 argn++;
	 if (!strcmp(argv[argn], "bounce"))
	    mode = BOUNCE;
	 else if (!strcmp(argv[argn], "left"))
	    mode = ROLL_LEFT;
	 else if (!strcmp(argv[argn], "right"))
	    mode = ROLL_RIGHT;
	 else if (!strcmp(argv[argn], "random"))
	    mode = RANDOM;
	 else if (!strcmp(argv[argn], "mode"))
	    random_mode = 1;
	 else
	    led_error ("Unknown Mode : ", argv[argn]);
	 break;
      case 'l':
	 load (atoi(argv[++argn]));
	 exit (0);
      default:
	 led_error ("Unkown Flag : ", argv[argn]);
      }
   }
   if (argn < argc)
      led_error ("Unknown Option : ", argv[argn]);

   while (1)
   {
      if (random_mode)
	 mode = rand () % N_MODES;
      switch (mode)
      {
      case BOUNCE:
	 bounce (time_between);
	 break;
      case ROLL_LEFT:
	 roll (time_between, 1, 5, 1);
	 break;
      case ROLL_RIGHT:
	 roll (time_between, 4, 0, -1);
	 break;
      case RANDOM:
	 random (time_between);
	 break;
      }
   }
}

